@@ -1,8 +1,181 @@
-#include "./test_group.hpp"
+#include "./test_lib/test_group.hpp"
+#include "./test_lib/test_group.def.hpp"
 
 namespace c4 {
 namespace yml {
 
+TEST(github, 475_0_space)
+{
+    Tree t;
+    ExpectError::check_success(&t, [&t]{
+        parse_in_arena(R"(
+test:
+- {a: 1}
+# next line has a trailing space
+- {b: 2} 
+# next line has a trailing space
+- [0, {c: 3}] 
+)", &t);
+    });
+    ConstNodeRef test = t["test"];
+    ASSERT_TRUE(test.is_seq());
+    ASSERT_EQ(test.num_children(), 3u);
+}
+
+TEST(github, 475_1_space_indented)
+{
+    Tree t;
+    ExpectError::check_success(&t, [&t]{
+        parse_in_arena(R"(
+test:
+  - {a: 1}
+  # next line has a trailing space
+  - {b: 2} 
+  # next line has a trailing space
+  - [0, {c: 3}] 
+)", &t);
+    });
+    ConstNodeRef test = t["test"];
+    ASSERT_TRUE(test.is_seq());
+    ASSERT_EQ(test.num_children(), 3u);
+}
+
+TEST(github, 475_2_tab)
+{
+    Tree t;
+    ExpectError::check_success(&t, [&t]{
+        parse_in_arena(R"(
+test:
+- {a: 1}
+# next line has a trailing tab
+- {b: 2}	
+# next line has a trailing tab
+- [0, {c: 3}]	
+)", &t);
+    });
+    ConstNodeRef test = t["test"];
+    ASSERT_TRUE(test.is_seq());
+    ASSERT_EQ(test.num_children(), 3u);
+}
+
+TEST(github, 475_3_tab_indented)
+{
+    Tree t;
+    ExpectError::check_success(&t, [&t]{
+        parse_in_arena(R"(
+test:
+  - {a: 1}
+  # next line has a trailing tab
+  - {b: 2}	
+  # next line has a trailing tab
+  - [0, {c: 3}]	
+)", &t);
+    });
+    ConstNodeRef test = t["test"];
+    ASSERT_TRUE(test.is_seq());
+    ASSERT_EQ(test.num_children(), 3u);
+}
+
+TEST(github, 475_4_space_map)
+{
+    Tree t;
+    ExpectError::check_success(&t, [&t]{
+        parse_in_arena(R"(
+test:
+  0: {a: 1}
+  # next line has a trailing space
+  1: {b: 2} 
+  # next line has a trailing space
+  2: [0, {c: 3}] 
+)", &t);
+    });
+    ConstNodeRef test = t["test"];
+    ASSERT_TRUE(test.is_map());
+    ASSERT_EQ(test.num_children(), 3u);
+}
+
+TEST(github, 475_5_tab_map)
+{
+    Tree t;
+    ExpectError::check_success(&t, [&t]{
+        parse_in_arena(R"(
+test:
+  0: {a: 1}
+  # next line has a trailing tab
+  1: {b: 2}	
+  # next line has a trailing tab
+  2: [0, {c: 3}]	
+)", &t);
+    });
+    ConstNodeRef test = t["test"];
+    ASSERT_TRUE(test.is_map());
+    ASSERT_EQ(test.num_children(), 3u);
+}
+
+
+//-----------------------------------------------------------------------------
+
+TEST(github, 455_0_ok)
+{
+    Tree t;
+    ExpectError::check_success([&t]{
+        parse_in_arena(R"(
+  processors:
+    - simple: a
+  sampler:
+    trace_id_ratio_based:
+      ratio:
+)", &t);
+    });
+    ConstNodeRef sampler = t["sampler"];
+    ASSERT_TRUE(sampler.is_map());
+    ASSERT_TRUE(sampler.has_child("trace_id_ratio_based"));
+    ASSERT_TRUE(sampler["trace_id_ratio_based"].has_child("ratio"));
+}
+
+TEST(github, 455_0)
+{
+    Tree t;
+    ExpectError::check_success([&t]{
+        parse_in_arena(R"(
+  processors:
+    - simple:
+  sampler:
+    trace_id_ratio_based:
+      ratio:
+)", &t);
+    });
+    ConstNodeRef sampler = t["sampler"];
+    ASSERT_TRUE(sampler.is_map());
+    ASSERT_TRUE(sampler.has_child("trace_id_ratio_based"));
+    ASSERT_TRUE(sampler["trace_id_ratio_based"].has_child("ratio"));
+}
+
+TEST(github, 455_1)
+{
+    Tree t;
+    ExpectError::check_success([&t]{
+        parse_in_arena(R"(
+file_format: 0.0
+tracer_provider:
+  processors:
+    - simple:
+        exporter:
+          console:
+  sampler:
+    trace_id_ratio_based:
+      ratio:
+    )", &t);
+    });
+    ConstNodeRef sampler = t["tracer_provider"]["sampler"];
+    ASSERT_TRUE(sampler.is_map());
+    ASSERT_TRUE(sampler.has_child("trace_id_ratio_based"));
+    ASSERT_TRUE(sampler["trace_id_ratio_based"].has_child("ratio"));
+}
+
+
+//-----------------------------------------------------------------------------
+
 TEST(github, 268)
 {
     Tree tree = parse_in_arena(R"(
@@ -12,7 +185,7 @@ TEST(github, 268)
             node: *bar
     )");
     tree.resolve();
-    auto root = tree.rootref();
+    ConstNodeRef root = tree.crootref();
     ASSERT_TRUE(root["map"].is_map());
     ASSERT_TRUE(root["map"].has_child("node"));
     ASSERT_EQ(root["map"]["node"], "bar");
@@ -31,11 +204,18 @@ TEST(github, 277)
     )");
     const char *keys[] = {"V", "W", "X"};
     const char *vals[] = {"5", "4", "6"};
+    #ifdef RYML_DBG
+    print_tree("parsed", tree);
+    #endif
+    test_invariants(tree);
     tree.resolve();
-    auto root = tree.rootref();
+    #ifdef RYML_DBG
+    print_tree("resolved", tree);
+    #endif
+    ConstNodeRef root = tree.crootref();
     ASSERT_TRUE(root["B"].is_map());
-    size_t num_childs = root["B"].num_children();
-    size_t child = 0;
+    id_type num_childs = root["B"].num_children();
+    id_type child = 0;
     ASSERT_EQ(num_childs, 3);
     for (const auto node : root["B"].children())
     {
@@ -46,7 +226,7 @@ TEST(github, 277)
     // test whether the tree is corrupted
     test_invariants(tree);
     child = num_childs;
-    for (size_t n = tree.last_child(root["B"].id()); n != NONE; n = tree.prev_sibling(n))
+    for (id_type n = tree.last_child(root["B"].id()); n != NONE; n = tree.prev_sibling(n))
     {
         ASSERT_NE(child, 0);
         EXPECT_EQ(tree.key(n), csubstr(keys[child - 1], 1));
@@ -175,13 +355,13 @@ TEST(github, 31)
     plist |= SEQ;
 
     {
-        auto lumi = plist.append_child();
+        NodeRef lumi = plist.append_child();
         lumi << "Lumi";
         EXPECT_TRUE(lumi.is_val());
     }
 
     {
-        auto lumi = plist.append_child();
+        NodeRef lumi = plist.append_child();
         lumi |= MAP;
         lumi["value"] << 1;
         lumi["relErr"] << 0.1;
@@ -190,7 +370,7 @@ TEST(github, 31)
 
     {
         ExpectError::check_assertion(&tree, [&](){
-            auto lumi = plist.append_child();
+            NodeRef lumi = plist.append_child();
             lumi << "Lumi";
             lumi |= MAP;
         });
@@ -198,7 +378,7 @@ TEST(github, 31)
 
     {
         ExpectError::check_assertion(&tree, [&](){
-            auto lumi = plist.append_child();
+            NodeRef lumi = plist.append_child();
             lumi << "Lumi";
             lumi |= SEQ;
         });
@@ -206,7 +386,7 @@ TEST(github, 31)
 
     {
         ExpectError::check_assertion(&tree, [&](){
-            auto lumi = plist.append_child();
+            NodeRef lumi = plist.append_child();
             lumi |= MAP;
             lumi << "Lumi";
         });
@@ -224,7 +404,7 @@ CASE_GROUP(GITHUB_ISSUES)
 ADD_CASE_TO_GROUP("github3-problem1",
 R"(
 translation: [-2, -2, 5])",
-L{N("translation", L{N("-2"), N("-2"), N("5")})}
+N(MB, L{N(KP|SFS, "translation", L{N(VP, "-2"), N(VP, "-2"), N(VP, "5")})})
 );
 
 // these must work without quotes
@@ -232,15 +412,16 @@ ADD_CASE_TO_GROUP("github3-problem2-ex1",
 R"(
 audio resource:
 )",
-L{N(KEYVAL, "audio resource", /*"~"*/{})}
+N(MB, L{N(KP|VN, "audio resource", /*"~"*/{})})
 );
+
 ADD_CASE_TO_GROUP("github3-problem2-ex2",
 R"(
 audio resource:
 more:
   example: y
 )",
-L{N(KEYVAL, "audio resource", /*"~"*/{}), N("more", L{N("example", "y")})}
+N(MB, L{N(KP|VN, "audio resource", /*"~"*/{}), N(KP|MB, "more", L{N(KP|VP, "example", "y")})})
 );
 
 ADD_CASE_TO_GROUP("github3-problem3",
@@ -250,12 +431,13 @@ R"(component:
   data:
     {}           # but this was not working
 )",
-L{N("component", L{
-        N("type", "perspective camera component"),
-        N(KEYMAP, "some_data", L{}),
-        N(KEYMAP, "data", L{})
+N(MB, L{
+  N(KP|MB, "component", L{
+    N(KP|VP, "type", "perspective camera component"),
+    N(KP|MFS, "some_data", L{}),
+    N(KP|MFS, "data", L{})
     }
-)}
+)})
 );
 
 /* THIS IS CAUSING VS TO CRASH OUT OF HEAP SPACE
@@ -378,11 +560,11 @@ R"(
   - 2.mp4
   - 3.mp4
 )",
-L{
-N(L{N("UQxRibHKEDI", L{N("0.mp4"), N("1.mp4"), N("2.mp4"), N("3.mp4")})}),
-N(L{N("DcYsg8VFdC0", L{N("0.mp4"), N("1.mp4"), N("2.mp4"), N("3.mp4")})}),
-N(L{N("Yt3ymqZXzLY", L{N("0.mp4"), N("1.mp4"), N("2.mp4"), N("3.mp4")})}),
-}
+N(SB, L{
+N(MB, L{N(KP|SB, "UQxRibHKEDI", L{N(VP,"0.mp4"), N(VP,"1.mp4"), N(VP,"2.mp4"), N(VP,"3.mp4")})}),
+N(MB, L{N(KP|SB, "DcYsg8VFdC0", L{N(VP,"0.mp4"), N(VP,"1.mp4"), N(VP,"2.mp4"), N(VP,"3.mp4")})}),
+N(MB, L{N(KP|SB, "Yt3ymqZXzLY", L{N(VP,"0.mp4"), N(VP,"1.mp4"), N(VP,"2.mp4"), N(VP,"3.mp4")})}),
+})
 );
 
 ADD_CASE_TO_GROUP("github6",
@@ -403,11 +585,13 @@ R"(videos:
   - 2.mp4
   - 3.mp4
 )",
-L{N("videos", L{
-N(L{N("UQxRibHKEDI", L{N("0.mp4"), N("1.mp4"), N("2.mp4"), N("3.mp4")})}),
-N(L{N("DcYsg8VFdC0", L{N("0.mp4"), N("1.mp4"), N("2.mp4"), N("3.mp4")})}),
-N(L{N("Yt3ymqZXzLY", L{N("0.mp4"), N("1.mp4"), N("2.mp4"), N("3.mp4")})}),
-})}
+N(MB, L{
+  N(KP|SB, "videos", L{
+    N(MB, L{N(KP|SB, "UQxRibHKEDI", L{N(VP,"0.mp4"), N(VP,"1.mp4"), N(VP,"2.mp4"), N(VP,"3.mp4")})}),
+    N(MB, L{N(KP|SB, "DcYsg8VFdC0", L{N(VP,"0.mp4"), N(VP,"1.mp4"), N(VP,"2.mp4"), N(VP,"3.mp4")})}),
+    N(MB, L{N(KP|SB, "Yt3ymqZXzLY", L{N(VP,"0.mp4"), N(VP,"1.mp4"), N(VP,"2.mp4"), N(VP,"3.mp4")})}),
+  })
+})
 );
 
 ADD_CASE_TO_GROUP("github34/ex1",
@@ -425,18 +609,18 @@ MessageID8:   "MapRegion_HyrulePrairie"
 MessageID9:          'MapRegion_HyrulePrairie'
 MessageID0:          "MapRegion_HyrulePrairie"
 )",
-L{
-  N(QV, "MessageID1", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID2", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID3", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID4", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID5", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID6", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID7", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID8", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID9", "MapRegion_HyrulePrairie"),
-  N(QV, "MessageID0", "MapRegion_HyrulePrairie"),
-}
+N(MB, L{
+  N(KP|VS, "MessageID1", "MapRegion_HyrulePrairie"),
+  N(KP|VD, "MessageID2", "MapRegion_HyrulePrairie"),
+  N(KP|VS, "MessageID3", "MapRegion_HyrulePrairie"),
+  N(KP|VD, "MessageID4", "MapRegion_HyrulePrairie"),
+  N(KP|VS, "MessageID5", "MapRegion_HyrulePrairie"),
+  N(KP|VD, "MessageID6", "MapRegion_HyrulePrairie"),
+  N(KP|VS, "MessageID7", "MapRegion_HyrulePrairie"),
+  N(KP|VD, "MessageID8", "MapRegion_HyrulePrairie"),
+  N(KP|VS, "MessageID9", "MapRegion_HyrulePrairie"),
+  N(KP|VD, "MessageID0", "MapRegion_HyrulePrairie"),
+})
 );
 
 ADD_CASE_TO_GROUP("github34/ex2",
@@ -454,18 +638,18 @@ R"(
 -  MessageID9:          'MapRegion_HyrulePrairie'
 -  MessageID0:          "MapRegion_HyrulePrairie"
 )",
-L{
-  N(L{N(QV, "MessageID1", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID2", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID3", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID4", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID5", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID6", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID7", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID8", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID9", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID0", "MapRegion_HyrulePrairie")}),
-}
+N(SB, L{
+  N(MB, L{N(KP|VS, "MessageID1", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID2", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VS, "MessageID3", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID4", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VS, "MessageID5", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID6", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VS, "MessageID7", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID8", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VS, "MessageID9", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID0", "MapRegion_HyrulePrairie")}),
+})
 );
 
 ADD_CASE_TO_GROUP("github34",
@@ -498,22 +682,22 @@ R"(
 - key2: true2
   MessageID2:          "MapRegion_HyrulePrairie2 "
 )",
-L{
-  N(L{N(QV, "MessageID1",  "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID2",  "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID3",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID4",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID5",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID6",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID7",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID8",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID9",  "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID10", "MapRegion_HyrulePrairie ")}),
-  N(L{N(QV, "MessageID11", "MapRegion_HyrulePrairie")}),
-  N(L{N(QV, "MessageID12", "MapRegion_HyrulePrairie")}),
-  N(L{N("key1", "true1"), N(QV, "MessageID1", "MapRegion_HyrulePrairie1 ")}),
-  N(L{N("key2", "true2"), N(QV, "MessageID2", "MapRegion_HyrulePrairie2 ")}),
-}
+N(SB, L{
+  N(MB, L{N(KP|VS, "MessageID1",  "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID2",  "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VS, "MessageID3",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VD, "MessageID4",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VS, "MessageID5",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VS, "MessageID6",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VD, "MessageID7",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VD, "MessageID8",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VS, "MessageID9",  "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VD, "MessageID10", "MapRegion_HyrulePrairie ")}),
+  N(MB, L{N(KP|VS, "MessageID11", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VD, "MessageID12", "MapRegion_HyrulePrairie")}),
+  N(MB, L{N(KP|VP, "key1", "true1"), N(KP|VS, "MessageID1", "MapRegion_HyrulePrairie1 ")}),
+  N(MB, L{N(KP|VP, "key2", "true2"), N(KP|VD, "MessageID2", "MapRegion_HyrulePrairie2 ")}),
+})
 );
 
 ADD_CASE_TO_GROUP("github35/expected_error11", EXPECT_PARSE_ERROR,
@@ -553,7 +737,9 @@ R"(
 );
 
 ADD_CASE_TO_GROUP("github128/1", RESOLVE_REFS | EXPECT_PARSE_ERROR, "a: *invalid");
-ADD_CASE_TO_GROUP("github128/2", RESOLVE_REFS/* | HAS_PARSE_ERROR*/, "*", N(DOCVAL, "*"));
+ADD_CASE_TO_GROUP("github128/2", RESOLVE_REFS | EXPECT_PARSE_ERROR, "*");
+ADD_CASE_TO_GROUP("github128/3", RESOLVE_REFS | EXPECT_PARSE_ERROR, "*abc");
+ADD_CASE_TO_GROUP("github128/4", "*abc", N(VAL, "*abc", AR(VALREF, "*abc")));
 
 ADD_CASE_TO_GROUP("github129", RESOLVE_REFS, R"(
 ref: &ref ref_val
@@ -572,19 +758,21 @@ h: |-     # don't resolve, it's just a string
   *ref
 i: |+     # don't resolve, it's just a string
   *ref
-)", L{
-        N("ref", "ref_val"),
-        N("a", "ref_val"),    // this should be resolved
-        N(QV, "b", "*ref"),   // this should not be resolved (just a string)
-        N(QV, "c", "*ref"),   // this should not be resolved (just a string)
-        N(QV, "d", "*ref\n"), // this should not be resolved (just a string)
-        N(QV, "e", "*ref"),   // this should not be resolved (just a string)
-        N(QV, "f", "*ref\n"), // this should not be resolved (just a string)
-        N(QV, "g", "*ref\n"), // this should not be resolved (just a string)
-        N(QV, "h", "*ref"),   // this should not be resolved (just a string)
-        N(QV, "i", "*ref\n"),   // this should not be resolved (just a string)
-    }
+)",
+N(MB, L{
+  N(KP|VP, "ref", "ref_val"),
+  N(KP|VP, "a", "ref_val"),    // this should be resolved
+  N(KP|VS, "b", "*ref"),   // this should not be resolved (just a string)
+  N(KP|VD, "c", "*ref"),   // this should not be resolved (just a string)
+  N(KP|VF, "d", "*ref\n"), // this should not be resolved (just a string)
+  N(KP|VF, "e", "*ref"),   // this should not be resolved (just a string)
+  N(KP|VF, "f", "*ref\n"), // this should not be resolved (just a string)
+  N(KP|VL, "g", "*ref\n"), // this should not be resolved (just a string)
+  N(KP|VL, "h", "*ref"),   // this should not be resolved (just a string)
+  N(KP|VL, "i", "*ref\n"),   // this should not be resolved (just a string)
+})
 );
+
 }
 
 } // namespace yml
